from __future__ import print_function

import os

from termcolor import cprint
from json import loads
from subprocess import check_output

import requests

def l(s, color=None, newline=True):
    ''' Print a string with a color, no endline.'''
    opts = {'end': '\n' if newline else ''}
    if color:
        cprint(s, color, **opts)
    else:
        print(s, **opts)



def get_repos():
    # set_trace()
    r = requests.get('https://api.github.com/orgs/lrn-guru/repos')
    repos = r.json()
    # repos = loads(open('~/repos.json', 'r').read())
    repo_names = []
    for repo in repos:
        name = repo['full_name'].split('/')
        project_name = name[1]
        if (project_name == 'lrn' or project_name == 'lrn-guru.github.io'):
            continue
        github_name = project_name[6:]
        js = get_config(github_name)
        description = js['short_description']
        difficulty = js['difficulty']
        repo_names.append((github_name, description, difficulty))

    return repo_names

def get_local_config():
    if os.path.exists('.config.json'):
        path = ''
    elif os.path.exists('../.config.json'):
        path = '../'
    elif os.path.exists('../../.config.json'):
        path = '../../'
    else:
        print('You are not currently in a tutorial directory.')
        exit(1)
    with open('{}.config.json'.format(path), 'r') as js:
        return loads(js.read())

def get_config(name):
    url = 'https://raw.github.com/lrn-guru/learn-{}/master/.config.json'.format(name)
    r = requests.get(url)
    return r.json()


def get_tests_dir():
    files = '.tests'
    for i in range(4):
        path = '../' * i + files
        if os.path.exists(path):
            return path

def get_task():
    branch = get_branch()
    config = get_local_config()
    for lesson in config['lessons']:
        if lesson['branch'] == branch:
            break # at the right lesson

    lrn_task = get_lrn_task()

    return lesson['tasks'][lrn_task]

def get_branch():
    status = check_output('git branch'.split())
    return status[2:status.find('\n')]

def get_starting_branch():
    config = get_local_config()
    return config['lessons'][0]['branch']

def find_lrn_txt():
    this_dir = '.lrn_task.txt'
    for i in range(4):
        path = '..' * i + this_dir
        if os.path.exists(path):
            return path

    # If it was not found, make it.
    return this_dir


def set_lrn_task(number):
    lrn_txt = find_lrn_txt()
    with open(lrn_txt, 'w') as f:
        f.write(str(number))

def get_lrn_task():
    lrn_txt = find_lrn_txt()
    with open(lrn_txt, 'r') as f:
        return int(f.read())



import os

try:
    from lrn.api import api
except:
    from api import api

l = api.l

from subprocess import check_output

def check_progress(cmd):
    task = api.get_task()
    test = task['test']
    test_dir = api.get_tests_dir()

    command = 'python {}/{}.py'.format(test_dir, test)
    status = check_output(command.split())
    return status

def repl():
    try:
        # os.environ['OLD_PS1'] = os.environ['PS1']
        # os.environ['PS1'] = 'lrn: ' + os.environ['PS1']
        while True:
            l('lrn ', 'cyan', False)
            l('> ', 'blue', False)
            cmd = raw_input('')
            os.system(cmd)
            outcome = check_progress(cmd)
            __import__('ipdb').set_trace()
            if outcome == "0":
                api.next()


    except EOFError:
        print('Exiting lrn...')
        exit(0)
        # os.environ['PS1'] = os.environ['OLD_PS1']



if __name__ == '__main__':
    repl()



#!/usr/bin/env python
import argparse
import sys
import os


import subprocess
from subprocess import call, check_output

try:
    from lrn.api import api
    from lrn.repl import repl
except:
    from api import api
    from repl import repl

l = api.l

projects = api.get_repos()


def list_projects():
    l('id'.ljust(9), 'white', False)
    l('description'.ljust(58), 'white', False)
    l('difficulty', 'white')
    l('-' * 73)
    for proj in projects:
        l(proj[0].ljust(9), 'yellow', False)
        l(proj[1].ljust(60), 'blue', False)
        l('*' * proj[2], 'yellow')

def introduce():
    """ Welcomes a new user to the {name} tutorial."""
    config = api.get_local_config()
    l(config['introduction'], 'green')


def task():
    """ Informs the user of their current task."""
    task = api.get_task()
    l(task['instruction'], 'cyan')



def start(name):
    url = 'https://github.com/lrn-guru/learn-{}.git'.format(name)
    # if there are no files, work here
    if len(check_output('ls -a'.split())) == 5:
        command = 'git clone {} .'.format(url)
    else:
        command = 'git clone {}'.format(url)
        # l('Enter `cd {}` to get started.'.format(name))

    dev_null = open(os.devnull, 'w')
    call(command.split(), stdout=dev_null, stderr=subprocess.STDOUT)
    api.set_lrn_task(0)

    branch = api.get_starting_branch()
    command = 'git checkout {}'.format(branch)
    call(command.split(), stdout=dev_null, stderr=subprocess.STDOUT)
    introduce()
    task()
    start_repl()


def run_tests():
    # get the current branch
    task = api.get_task()
    test = task['test']

    print(test)
    # test_file =

def hint():
    task_json = api.get_task()
    print(task_json['hint'])

def start_repl():
    """
    Put the user into an interactive session where
    we can monitor their output. They exit with control d.
    cd into the right directory if possible.
    """
    repl.repl()



def progress():
    #displaying completed or incompleted lessons
    #get local config to parse:

    ############# set up progress report #######################
    l('-' * 73)
    # cprint('Progress Report: \n\n', 'cyan')
    # word = colored('  Complete:   ', 'cyan')
    # c_text = colored('   ', 'green', attrs=['reverse', 'blink'])
    # print(word + '[' + c_text + ']')

    # word = colored('  Incomplete: ', 'cyan')
    # i_text = colored('   ', 'red', attrs=['reverse', 'blink'])
    # print(word + '[' + i_text + ']\n')
    ############# end set up              ######################

    full_local_config = api.get_local_config()
    if full_local_config == 1:
        return
    else:
        for j in full_local_config['lessons']:
            pass
            # text = colored(j['name'], 'blue')
            # print('  [' + i_text + ']: ' + text)
        # print('\n')

def main():
    if args.command == 'list':
        list_projects()
    elif args.command == 'start':
        if len(sys.argv) == 3:
            start(sys.argv[-1])
        else:
            l('Error: Specify a tutorial to start.', 'red')
    elif args.command == 'hint':
        hint()
    elif args.command == 'progress':
        progress()
    else:
        print('!')

if __name__ == '__main__':
    description = 'lrn -- the interactive tutorial platform'
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('command', help='The action to take (list, start, et cetera)')
    # parser.add_argument('-v', '--virtual', help='Whether or not to run lrn in a virtual machine')

    args, sub_args = parser.parse_known_args()
    main()



from distutils.core import setup

setup(
    name='lrn',
    version='0.1.0',
    author='Razzi Abuissa + Alex Brashear + Peter Yin',
    author_email='py.peteryin@gmail.com',
    url='https://github.com/lrn-guru',
    packages=['lrn', 'lrn.api', 'lrn.repl'],
    scripts=['lrn/lrn'],
)

